mydir="YOUR PATH HERE"
setwd(paste0(mydir,"Generate_Fig1b_ExtFig2_ExtFig6"))

require(ggplot2)
require(tseries)
Sys.setenv(TZ='UTC')
library(lubridate)
library(zoo)
library(dplyr) # for lag


####################
## ROLLregression ##
####################

## rolling regression line to get estimated rate of change (slope) 
## symmetric window,  x>=xout-width; x<=xout+width etc for width=window/2
rollregression_eq = function(x,y,xout,width) {
  out = numeric(length(xout))
  for( i in seq_along(xout) ) {
	tempy=y[(x >= (xout[i]-width)) & (x <= (xout[i]+width))]
	tempx=x[(x >= (xout[i]-width)) & (x <= (xout[i]+width))]
	tempmodel = lm(tempy ~ tempx)
	out[i] =tempmodel$coefficients[[2]]
	}

  return(out)
}


################
##  ROLLSUM  ##
################

## rolling sum with equal, symmetric window,  x>=xout-width; x<=xout+width etc for width=window/2
rollsum_eq = function(x,y,xout,width) {
  out = numeric(length(xout))
  for( i in seq_along(xout) ) out[i] = sum(y[(x >= (xout[i]-width)) & (x <= (xout[i]+width))], na.rm=TRUE)
  return(out)
}


########################
# Regular 1 Myr series #
########################
tt=seq(-1000,0,1)
TS_empty<-zoo(,tt)


################################
# Fragmentaion - FROM MERDITH #
################################

# file with 1Myr resolution data
# 1_Ga_area_perimeter_1000_0_1myr.csv 
#MerdithFrag<-read.csv("D:\\DATA\\TOM_KIMBERLITES\\data_TOM\\1_Ga_area_perimeter_1000_0_1myr.csv", 
#header=T, stringsAsFactors = F)
MerdithFrag<-read.csv(paste0(mydir,"input data files\\1_Ga_area_perimeter_1000_0_1myr.csv"), 
header=T, stringsAsFactors = F)

MerdithFrag$Age=-MerdithFrag$time
MerdithFrag=MerdithFrag[order(MerdithFrag$Age),]

MFrag1Myr_5e5= MerdithFrag$plate_perimeter_500000/MerdithFrag$plate_area_500000




################
# Kimberlites  #
################

#Kimberlites_In<-read.csv("D:\\DATA\\TOM_KIMBERLITES\\data_TOM\\Kimberlites_ages.csv", 
#header=T, stringsAsFactors = F)

Kimberlites_In<-read.csv(paste0(mydir,"input data files\\Kimberlites_ages.csv"), 
header=T, stringsAsFactors = F)
Kimberlites_In$Age=-Kimberlites_In$Age
Kimberlites_In=Kimberlites_In[order(Kimberlites_In$Age),]
KimberliteAge =Kimberlites_In$Age


################
# Preservation #
################

#Goodwin<-read.csv("D:\\DATA\\TOM_KIMBERLITES\\data_TOM\\GoodwinPreservationCurve.csv", 
#header=T, stringsAsFactors = F)
Goodwin<-read.csv(paste0(mydir,"input data files\\GoodwinPreservationCurve.csv"), 
header=T, stringsAsFactors = F)
Goodwin$Age =-Goodwin$Age_Ma
Goodwin=Goodwin[order(Goodwin$Age),]
preservation= approx( x=Goodwin$Age, y=Goodwin$Preservation, xout=tt, rule=2, ties= "mean")$y

##################################
# Kimberlite time series, ages rounded to 1 Myr 
minage=min(tt)
KimAge=round(KimberliteAge)
KimAge=KimAge[which(KimAge>=minage)]
KRzoo=zoo(1,order.by=KimAge)
KRzoo=aggregate(KRzoo, index(KRzoo), sum)
#merge with regular time series
TS_empty=zoo(, tt)
z <- merge.zoo(TS_empty,KRzoo)
z[which(is.na(z))]=0
KRzoo=z

## Correct rate of kimberlite eruptions by preservation factor
CorRKimberlites=KRzoo/preservation

########################################################################################
## testing ##
## look at correlation with rate of change of fragmentation
## using different window widths:
## 
win=2
dFrag_5e5_w4=rollregression_eq(tt,MFrag1Myr_5e5,tt,width=win) 
win=5
dFrag_5e5_w10=rollregression_eq(tt,MFrag1Myr_5e5,tt,width=win) 
win=10
dFrag_5e5_w20=rollregression_eq(tt,MFrag1Myr_5e5,tt,width=win) 
win=25
dFrag_5e5_w50=rollregression_eq(tt,MFrag1Myr_5e5,tt,width=win) 
ccf(coredata(CorRKimberlites),dFrag_5e5_w4,lag.max = 150)
ccf(coredata(CorRKimberlites),dFrag_5e5_w10,lag.max = 150)
ccf(coredata(CorRKimberlites),dFrag_5e5_w20,lag.max = 150)
ccf(coredata(CorRKimberlites),dFrag_5e5_w50,lag.max = 150)
# peak correlation with rate of change (dFrag) seems to be around 25Myr 
# window ~10 Myr gives good balance of smoothing noise and resolution
########################################################################################


# use a t+/-4Myr rolling window for good balance of smoothing and resolution
# also allows regular (non overlapping) offsets for future analysis

win=4
lagtt=tt[(1+win):(length(tt)-win)]
dFrag5e5_9Myr=rollregression_eq(tt,MFrag1Myr_5e5,lagtt,width=win) 
dFrag5e5_9Myr=c(rep(NA,win,),dFrag5e5_9Myr,rep(NA,win,))

dFrag9MyrZoo=zoo(dFrag5e5_9Myr,tt)
plot(dFrag9MyrZoo)

########################################################################################
#### plot all the timeseries correlations
#### INCLUDING just for past 500 Myr

x=data.frame(Age=tt, Frag=MFrag1Myr_5e5, dFrag_9Myr=dFrag5e5_9Myr,Kimberlites=coredata(KRzoo), CorrKimberlites=CorRKimberlites)

#write.table(x, "Symmetric_dFrag9Myr_alldataNA.csv", sep=",", row.names=F, quote=F)

x=na.omit(x)

x500Myr=x[which(x$Age>=-500),]
x200Myr=x[which(x$Age>=-200),]

MFrag=zoo(x=MFrag1Myr_5e5, tt)
plot(MFrag)

#write.table(x, "Symmetric_dFrag9Myr_alldata.csv", sep=",", row.names=F, quote=F)
#output everything at the end with LIP data included

####################################################
### Kimberlites and Frag RATE over 9Myr           ##
####################################################
temp=ccf(x$dFrag_9Myr,x$Kimberlites , lag.max=150)
myfile="ExFig2b_Mfrag_CCF_SymmetricdFrag_9Myr_Kimberlites_996-4Myr.pdf"
pdf(myfile, width=6,height=4)
ccf(x$dFrag_9Myr,x$Kimberlites , lag.max=150)
maxcorr=temp$acf[which(abs(temp$acf)==max(abs(temp$acf)))]
maxlag=temp$lag[which(temp$acf==maxcorr)]
text( x=100,y=maxcorr-0.05, labels = paste0("max = ", signif(maxcorr,digits=2)) )
text( x=100,y=maxcorr-0.1, labels = paste0("lag = ", signif(maxlag,digits=2)) )
dev.off()

temp=ccf(x200Myr$dFrag_9Myr,x200Myr$Kimberlites , lag.max=50)
myfile="ExFig2d_Mfrag_CCF_SymmetricdFrag_9Myr_Kimberlites_200-4Myr.pdf"
pdf(myfile, width=6,height=4)
ccf(x200Myr$dFrag_9Myr,x200Myr$Kimberlites , lag.max=50)
maxcorr=temp$acf[which(abs(temp$acf)==max(abs(temp$acf)))]
maxlag=temp$lag[which(temp$acf==maxcorr)]
text( x=20,y=maxcorr-0.05, labels = paste0("max = ", signif(maxcorr,digits=2)) )
text( x=20,y=maxcorr-0.1, labels = paste0("lag = ", signif(maxlag,digits=2)) )
dev.off()


####################################################
### CORRECTED Kimberlites & Frag RATE over 9Myr  ##
####################################################
temp=ccf(x$dFrag_9Myr,x$CorrKimberlites , lag.max=150)
text( x=100,y=max(temp$acf)-0.05, labels = paste0("max = ", signif(max(temp$acf),digits=2)) )
text( x=100,y=max(temp$acf)-0.1, labels = paste0("lag = ", signif(temp$lag[which(temp$acf==max(temp$acf))],digits=2)) )
myfile="ExFig2c_Mfrag_CCF_SymmetricdFrag_9Myr_CorrKimberlites_996-4Myr.pdf"
pdf(myfile, width=6,height=4)
ccf(x$dFrag_9Myr,x$CorrKimberlites , lag.max=150)
maxcorr=temp$acf[which(abs(temp$acf)==max(abs(temp$acf)))]
maxlag=temp$lag[which(temp$acf==maxcorr)]
text( x=100,y=maxcorr-0.05, labels = paste0("max = ", signif(maxcorr,digits=2)) )
text( x=100,y=maxcorr-0.1, labels = paste0("lag = ", signif(maxlag,digits=2)) )
dev.off()

##############################
temp=ccf(x500Myr$dFrag_9Myr,x500Myr$CorrKimberlites , lag.max=150)

maxcorr=temp$acf[which(abs(temp$acf)==max(abs(temp$acf)))]
maxlag=temp$lag[which(temp$acf==maxcorr)]
text( x=100,y=maxcorr+0.05, labels = paste0("max = ", signif(maxcorr,digits=2)) )
text( x=100,y=maxcorr+0.1, labels = paste0("lag = ", signif(maxlag,digits=2)) )

myfile="Fig1b_Mfrag_CCF_SymmetricdFrag_9Myr_CorrKimberlites_500-4Myr.pdf"
pdf(myfile, width=6,height=4)
ccf(x500Myr$dFrag_9Myr,x500Myr$CorrKimberlites , lag.max=150)
maxcorr=temp$acf[which(abs(temp$acf)==max(abs(temp$acf)))]
maxlag=temp$lag[which(temp$acf==maxcorr)]
text( x=100,y=maxcorr-0.05, labels = paste0("max = ", signif(maxcorr,digits=2)) )
text( x=100,y=maxcorr-0.1, labels = paste0("lag = ", signif(maxlag,digits=2)) )
dev.off()




##############
################################################


################
# LIP events   #
################

LIP_In<-read.csv(paste0(mydir,"input data files\\LIP_input.csv"), 
header=T, stringsAsFactors = F)

LIP_In$AGE =-LIP_In$AGE
LIP_In$AGE_TO =-LIP_In$AGE_TO
LIP_In=LIP_In[order(LIP_In$AGE),]
LIP_In$AGE_TO[which(is.na(LIP_In$AGE_TO))]=LIP_In$AGE[which(is.na(LIP_In$AGE_TO))]

# Round to get simple regular timeseries
LIP_In$AGE = round(LIP_In$AGE)
LIP_In$AGE_TO =round(LIP_In$AGE_TO)

for (i in 1:length(LIP_In$AGE)) {
temp=seq(LIP_In$AGE[i],LIP_In$AGE_TO[i],1)
if(i==1) allages=temp else allages=c(allages,temp)
}

allages<-allages[order(allages)]
LIP_zoo=zoo(1,order.by=allages)
LIP_zoo=aggregate(LIP_zoo, index(LIP_zoo), sum)

#TS_empty<-zoo(,seq(min(allages),max(allages),1))
TS_empty<-zoo(,seq(min(allages),0,1))
z <- merge.zoo(TS_empty,LIP_zoo)
z[which(is.na(z))]=0

## note this counts every active LIP at a given time step (1 Myr)
LIP_zoo=window(z, start=-1000, end=0)

## now just get series with startage only
allages<- LIP_In$AGE[order(LIP_In$AGE)]
LIPstart_zoo=zoo(1,order.by=allages)
LIPstart_zoo=aggregate(LIPstart_zoo, index(LIPstart_zoo), sum)

TS_empty<-zoo(,seq(min(allages),0,1))
z <- merge.zoo(TS_empty,LIPstart_zoo)
z[which(is.na(z))]=0

LIPstart_zoo=window(z, start=-1000, end=0) # just use up to 1000 Ma

################
# use the LIP timseries to get time range
################
tt=index(LIPstart_zoo)
TS_empty<-zoo(,tt)
################

# use dFrag timeseries to set limits
start=min(x$Age)
end=max(x$Age)

t=ccf(window(dFrag9MyrZoo,start=start, end=end), window(LIPstart_zoo, start=start, end=end), lag.max=150)
maxcorr=t$acf[which(abs(t$acf)==max(abs(t$acf)))]
maxlag=t$lag[which(abs(t$acf)==max(abs(t$acf)))]
text( x=100,y=0.13, labels = paste0("max = ", signif(maxcorr,digits=2)), adj=0 )
text( x=100,y=0.11, labels = paste0("lag = ", signif(maxlag,digits=2)), adj=0)

myfile="ExFig6a_CCF_DFRAG9Myr_LIPstart_ALLDATA.pdf"
pdf(myfile, width=6, height=4)
ccf(window(dFrag9MyrZoo,start=start, end=end), window(LIPstart_zoo, start=start, end=end), lag.max=150)
text( x=100,y=0.13, labels = paste0("max = ", signif(maxcorr,digits=2)), adj=0 )
text( x=100,y=0.11, labels = paste0("lag = ", signif(maxlag,digits=2)), adj=0)
dev.off()



## Merge with kimberlite data to get single data frame to output
x=data.frame(Age=tt, Frag=MFrag1Myr_5e5, dFrag_9Myr=dFrag5e5_9Myr,Kimberlites=coredata(KRzoo), 
CorrKimberlites=CorRKimberlites, LIPstart=coredata(LIPstart_zoo))

write.table(x, "Symmetric_dFrag_5_9Myr_alldataNA.csv", sep=",", row.names=F, quote=F)

x=na.omit(x)

x500Myr=x[which(x$Age>=-500),]
x200Myr=x[which(x$Age>=-200),]

MFrag=zoo(x=MFrag1Myr_5e5, tt)
plot(MFrag)

write.table(x, "Symmetric_dFrag9Myr_alldata.csv", sep=",", row.names=F, quote=F)


###########################################################################
###########################################################################
########      Get data for Uninet BN (5 Myr resolution bins)       ########
###########################################################################
###########################################################################

## calculate rolling sum of kimberlites and lip events in 5 Myr bins 
## centered, t +/- 2 Myr

win=1
hwidth=2
maxlag=50
lagtt=tt[(1+hwidth):(length(tt)-hwidth)]
len=length(tt)

KIM_rollsum5Myr_lag0=rollsum_eq(tt,coredata(KRzoo),lagtt,width=hwidth) 
KIM_rollsum5Myr_lag0=c(rep(NA,hwidth),KIM_rollsum5Myr_lag0,rep(NA,hwidth))

LIPstart_rollsum5Myr_lag0=rollsum_eq(tt,coredata(LIPstart_zoo),lagtt,width=hwidth) 
LIPstart_rollsum5Myr_lag0=c(rep(NA,hwidth),LIPstart_rollsum5Myr_lag0,rep(NA,hwidth))

LIPstart_rollsum5Myr=zoo(LIPstart_rollsum5Myr_lag0,tt)
KIM_rollsum5Myr=zoo(KIM_rollsum5Myr_lag0,tt)

test=ccf(na.omit(dFrag9MyrZoo), na.omit(LIPstart_rollsum5Myr), lag.max = 150)
test$lag[which(test$acf==max(test$acf))]

test2=ccf(na.omit(dFrag9MyrZoo), na.omit(KIM_rollsum5Myr), lag.max = 150)
test2$lag[which(test2$acf==max(test2$acf))]

plot(test)
plot(test2)



# just take values every 5 Myr (reducing resolution, no overlap)
plot(LIPstart_rollsum5Myr[seq(3,1000,5),])
plot(KIM_rollsum5Myr[seq(3,1000,5),])
plot(dFrag9MyrZoo[seq(3,1000,5),])

#merge into one:
merge5myrRes=merge.zoo(KIM_rollsum5Myr[seq(3,1000,5),], LIPstart_rollsum5Myr[seq(3,1000,5),],dFrag9MyrZoo[seq(3,1000,5),])

names(merge5myrRes)=c("KIM_rollsum5Myr","LIPstart_rollsum5Myr","dFrag_5e5_9Myr")

# make a timeseries with offset (leading) values for LIPstart and dFrag to 
# analyse lags in Uninet:
uninet_input=data.frame(Age=index(merge5myrRes))
uninet_input=cbind(uninet_input, coredata(merge5myrRes))

tlen=length(index(merge5myrRes))
for (i in 1:10) {
	temp= coredata(merge5myrRes)[1:(tlen-i),]
	temp=rbind(matrix(NA,nrow=i,ncol=3), temp)
	temp=as.data.frame(temp)

	names(temp)=paste0(names(temp),"_lag",(i*5))
	uninet_input=cbind(uninet_input, temp)
}

write.csv(na.omit(uninet_input), "UNINET_KimberliteCount_LIPstart_dFrag_5MyrRes.csv", row.names=F, quote=F)
#write.csv(names(uninet_input), "ALLNodenames_KimberliteCount_LIPstart_dFrag_5MyrRes.csv", row.names=F, quote=F)

uninet_input500Myr=uninet_input[which(uninet_input$Age>=-500),]
write.table(na.omit(uninet_input500Myr),  "UNINET_Past500Myr_KimberliteCount_LIPstart_dFrag_5MyrRes.csv" , sep=",", row.names=F, quote=F)
